import re


class ValidationError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class AsciiTerminal:
    """
    Provides a convenient way of representing the terminal as text in the tests e.g. a terminal of 5x5 is represented
    as:

    +-----+
    |     |
    |     |
    |     |
    |     |
    |     |
    +-----+
    """

    _WIDTH_REGEX = re.compile("\s*\+([-]+)\+")

    @staticmethod
    def _strip_chars_outside_terminal_border(target_lines):
        spaces_stripped = map(lambda x: x.strip(" "), target_lines)
        non_empty = filter(lambda l: len(l) > 0, spaces_stripped)
        return list(non_empty)

    @staticmethod
    def _validate(lines):
        def any_filtered(filter_func, items):
            results = list(filter(filter_func, items))
            return len(results) > 0

        if len(lines) < 2:
            raise ValidationError("ASCII terminal must contain at width indicator at top and bottom")

        if not AsciiTerminal._WIDTH_REGEX.match(lines[0]):
            raise ValidationError("First line must be a width indicator")

        if not AsciiTerminal._WIDTH_REGEX.match(lines[-1]):
            raise ValidationError("Last line must be a width indicator")

        if len(lines) > 2:
            console_lines = lines[1:-1]

            if any_filtered(lambda line: len(line) < 2, console_lines):
                raise ValidationError("All lines between width indicators contain at least two pipes")

            if any_filtered(lambda line: line[0] != "|", console_lines):
                raise ValidationError("All lines between width indicators must start with a pipe")

            if any_filtered(lambda line: line[-1] != "|", console_lines):
                raise ValidationError("All lines between width indicators must end with a pipe")

    @staticmethod
    def _remove_border(lines):
        without_width_indicators = lines[1:-1]
        without_pipes = map(lambda line: line[1:-1], without_width_indicators)
        text = "\n".join(list(without_pipes))
        return text.rstrip("\n")

    @staticmethod
    def extract_text(text):
        lines = text.split("\n")
        cleaned = AsciiTerminal._strip_chars_outside_terminal_border(lines)
        AsciiTerminal._validate(cleaned)
        return AsciiTerminal._remove_border(cleaned)



import click
import unittest
from click.testing import CliRunner
from doodledashboard.notifications import TextNotification
from parameterized import parameterized

from sketchingdev.console import ConsoleDisplay
from tests.sketchingdev.terminal.ascii_terminal import AsciiTerminal


class TestConsoleDisplayWithText(unittest.TestCase):

    @parameterized.expand([
        ((1, 1), "",
         """
         +-+
         ||
         +-+
         """),
        ((10, 3), "a",
         """
         +----------+
         ||
         |    a|
         ||
         +----------+
         """),
        ((10, 3), "centred",
         """
         +----------+
         ||
         | centred|
         ||
         +----------+
         """),
        ((10, 3), "I'm centred",
         """
         +----------+
         |   I'm|
         | centred|
         ||
         +----------+
         """),
        ((10, 3), "Hello World! This is too long",
         """
         +----------+
         |  Hello|
         |  World!|
         | This is|
         +----------+
         """),
    ])
    def test_text_centred_in_console(self, console_size, input_text, expected_ascii_terminal):
        expected_terminal = AsciiTerminal.extract_text(expected_ascii_terminal)

        text_notification = TextNotification()
        text_notification.set_text(input_text)
        cmd = create_cmd(lambda: ConsoleDisplay(console_size).draw(text_notification))
        result = CliRunner().invoke(cmd, catch_exceptions=False)

        self.assertEqual(expected_terminal, result.output)


def create_cmd(func):
    @click.command()
    def c(f=func):
        f()

    return c


if __name__ == "__main__":
    unittest.main()



import math


def calculate_padding_to_centre(total_space, used_space):
    if used_space is 0:
        return 0

    padding = total_space - used_space
    return math.floor(padding / 2) if padding > 0 else 0


def centre_in_container(lines, size):
    def pad_text_left(width, line):
        padding = " " * calculate_padding_to_centre(width, len(line))
        return padding + line

    def pad_lines_top(height, lines):
        result = []
        height_padding = calculate_padding_to_centre(height, len(lines))
        for _ in range(height_padding):
            result.insert(0, "")

        return result + lines

    centred_horizontally = list(map(lambda line: pad_text_left(size[0], line), lines))
    centred_vertically = pad_lines_top(size[1], centred_horizontally)

    return centred_vertically



import click
from doodledashboard.configuration.config import ConfigSection
from doodledashboard.display import Display
from doodledashboard.notifications import TextNotification, ImageNotification

from sketchingdev.notifications.image import format_image
from sketchingdev.notifications.text import format_text


class ConsoleDisplay(Display):

    _NOTIFICATIONS = {
        TextNotification: format_text,
        ImageNotification: format_image
    }

    def __init__(self, size=click.get_terminal_size()):
        self._size = size

    def draw(self, notification):
        click.clear()
        factory = self.find_factory(notification)
        click.echo(factory(self._size, notification), nl=False)

    def find_factory(self, notification, default=lambda x, y: ""):
        for factory_type, factory in self._NOTIFICATIONS.items():
            if isinstance(notification, factory_type):
                return factory

        return default

    @staticmethod
    def get_supported_notifications():
        return ConsoleDisplay._NOTIFICATIONS.keys()

    @staticmethod
    def get_id():
        return "console"

    def __str__(self):
        return "Console display"

    @staticmethod
    def get_config_factory():
        return ConsoleConfig()


class ConsoleConfig(ConfigSection):

    @property
    def id_key_value(self):
        return "display", "console"

    def create(self, config_section):
        return ConsoleDisplay()



import click
import os
import unittest
from click.testing import CliRunner
from doodledashboard.notifications import ImageNotification
from os import path
from parameterized import parameterized

from sketchingdev.console import ConsoleDisplay
from tests.sketchingdev.terminal.ascii_terminal import AsciiTerminal


def _get_current_directory():
    return os.path.dirname(os.path.realpath(__file__))


def resolve_data_path(filename):
    data_dir = path.join(_get_current_directory(), "data/")
    return path.join(data_dir, filename)


class TestConsoleDisplayWithImages(unittest.TestCase):

    @parameterized.expand([
        ((10, 10), resolve_data_path("cross-without-transparency.png"),
         """
         +----------+
         |   @@@%   |
         |   @@%#   |
         |   @%##   |
         |@@@%##*=-:|
         |@@%##*=-::|
         |@%##*=-::,|
         |%##*=-::,.|
         |   =-::   |
         |   -::,   |
         |   ::,.   |
         +----------+
         """),
        ((10, 10), resolve_data_path("cross-with-transparency.png"),
         """
         +----------+
         |   @@@%   |
         |   @@%#   |
         |   @%##   |
         |@@@%##*=-:|
         |@@%##*=-::|
         |@%##*=-::,|
         |%##*=-::,.|
         |   =-::   |
         |   -::,   |
         |   ::,.   |
         +----------+
         """),
        ((12, 12), resolve_data_path("cross-without-transparency.png"),
         """
         +-----------+
         ||
         |    @@@%   |
         |    @@%#   |
         |    @%##   |
         | @@@%##*=-:|
         | @@%##*=-::|
         | @%##*=-::,|
         | %##*=-::,.|
         |    =-::   |
         |    -::,   |
         |    ::,.   |
         ||
         +-----------+
         """),
        ((10, 10), resolve_data_path("rgb-with-transparency.png"),
         """
         +----------+
         |          |
         | ####==== |
         | ####==== |
         | ####==== |
         | ####==== |
         |   ####   |
         |   ####   |
         |   ####   |
         |   ####   |
         |          |
         +----------+
         """),
        ((10, 5), resolve_data_path("3x3-box.png"),
         """
         +----------+
         ||
         |   @@@|
         |   @@@|
         |   @@@|
         ||
         +----------+
         """)
    ])
    def test_text_centred_in_console(self, console_size, input_image, expected_ascii_terminal):
        expected_terminal = AsciiTerminal.extract_text(expected_ascii_terminal)

        image_notification = ImageNotification()
        image_notification.set_image_path(input_image)
        cmd = create_cmd(lambda: ConsoleDisplay(console_size).draw(image_notification))
        result = CliRunner().invoke(cmd, catch_exceptions=False)

        self.assertEqual(expected_terminal, result.output)


def create_cmd(func):
    @click.command()
    def c(f=func):
        f()

    return c


if __name__ == "__main__":
    unittest.main()



from PIL import Image
from sketchingdev.notifications.centre import centre_in_container

GREYSCALE_CHARS = ['@', '%', '#', '*', '+', '=', '-', ':', ',', '.', ' ']
GREYSCALE_CHARS_MAX_INDEX = len(GREYSCALE_CHARS) - 1
ELEMENT_RANGE = element_range = GREYSCALE_CHARS_MAX_INDEX / 255


def map_pixel_to_character(pixel):
    char_index = int(ELEMENT_RANGE * pixel)
    return GREYSCALE_CHARS[char_index]


def convert_to_grayscale(image):
    return image.convert('L')


def convert_image_to_ascii(image):
    image = convert_to_grayscale(image)

    width, height = image.size
    pixels = list(image.getdata())
    rows = [pixels[i * width:(i + 1) * width] for i in range(height)]

    ascii_rows = []
    for row in rows:
        textual_row = "".join([map_pixel_to_character(pixel) for pixel in row])

        ascii_rows.append(textual_row)

    return ascii_rows


def format_image(size, notification):
    image_path = notification.get_image_path()

    try:
        image = Image.open(image_path)
    except Exception:
        return "ERROR\nUnable to open image: %s" % notification.get_image_path()

    image.thumbnail(size, Image.ANTIALIAS)
    ascii_image = convert_image_to_ascii(image)
    centred_image = centre_in_container(ascii_image, size)

    return "\n".join(centred_image).rstrip("\n")



import click
import unittest
from click.testing import CliRunner

from sketchingdev.console import ConsoleDisplay


class TestConsoleDisplay(unittest.TestCase):

    def test_id(self):
        self.assertEqual("console", ConsoleDisplay().get_id())

    def test_nothing_drawn_for_unsupported_notifications(self):
        image_notification = None
        cmd = create_cmd(lambda: ConsoleDisplay().draw(image_notification))
        result = CliRunner().invoke(cmd, catch_exceptions=False)

        self.assertEqual("", result.output)


def create_cmd(func):
    @click.command()
    def c(f=func):
        f()

    return c


if __name__ == "__main__":
    unittest.main()



import pytest
import unittest

from tests.sketchingdev.terminal.ascii_terminal import ValidationError, AsciiTerminal


class AsciiTerminalTests(unittest.TestCase):

    def test_validation_fails_if_first_line_not_width_indicator(self):
        text_terminal = """

        |A
        +-+
        """

        with pytest.raises(ValidationError) as err_info:
            AsciiTerminal.extract_text(text_terminal)

        self.assertEqual("First line must be a width indicator", err_info.value.value)

    def test_validation_fails_if_last_line_not_width_indicator(self):
        text_terminal = """
        +-+
        |A
        """

        with pytest.raises(ValidationError) as err_info:
            AsciiTerminal.extract_text(text_terminal)

        self.assertEqual("Last line must be a width indicator", err_info.value.value)

    def test_validation_fails_if_lines_between_width_indicators_do_not_start_with_pipe(self):
        text_terminal = """
                +-+
                 A
                +-+
                """

        with pytest.raises(ValidationError) as err_info:
            AsciiTerminal.extract_text(text_terminal)

        self.assertEqual("All lines between width indicators contain at least two pipes", err_info.value.value)

    def test_validation_fails_if_lines_between_width_indicators_does_not_end_with_pipe(self):
        text_terminal = """
                +-+
                |A
                +-+
                """

        with pytest.raises(ValidationError) as err_info:
            AsciiTerminal.extract_text(text_terminal)

        self.assertEqual("All lines between width indicators must end with a pipe", err_info.value.value)

    def test_get_text_returns_single_line_text(self):
        text_terminal = """
                +----+
                |   A|
                +----+
                """

        terminal = AsciiTerminal.extract_text(text_terminal)
        self.assertEqual("   A", terminal)

    def test_get_text_returns_multiline_text(self):
        text_terminal = """
                +----+
                |   A|
                |  B|
                +----+
                """

        terminal = AsciiTerminal.extract_text(text_terminal)
        self.assertMultiLineEqual("   A\n  B", terminal)

    def test_get_text_returns_multiline_text_without_trailing_whitespace(self):
        text_terminal = """
                +----+
                |   A|
                |  B|
                ||
                +----+
                """

        terminal = AsciiTerminal.extract_text(text_terminal)
        self.assertEqual("   A\n  B", terminal)



import textwrap

from sketchingdev.notifications.centre import centre_in_container


def format_text(size, notification):
    text = notification.get_text()
    width = size[0]
    height = size[1]

    wrapped_lines = textwrap.wrap(text, width)
    cropped = wrapped_lines[:height]

    centred = centre_in_container(cropped, size)

    return "\n".join(centred)
